<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="text"  />

<xsl:template match="/">
	<xsl:apply-templates select="ccdDiagram"/>
</xsl:template>

<!--capire come gestire spazie a capo-->

<xsl:template match="ccdDiagram">
	module <xsl:value-of select = "name"/>
	<xsl:apply-templates select="components"/>
</xsl:template>

<xsl:template match = "components">
	<xsl:apply-templates select = "class"/> 
	<xsl:apply-templates select = "association"/>
	<xsl:call-template name ="tipi"/>
</xsl:template>

<xsl:template match = "class">
	
	<xsl:variable name = "classe"><xsl:value-of select = "name"/></xsl:variable>
	<xsl:choose>
	<xsl:when test = "count(//generalization/subClasses[subClass = concat(/ccdDiagram/name,'_',$classe)]) &lt;= 1"> <!--se la classe non appartiene --> 

	sig <xsl:value-of select = "name"/> 
		
	<xsl:call-template name="costruisciGeneralizzazione"> 
			<xsl:with-param name = "className" select = "name"/>
   	</xsl:call-template>
	</xsl:when>
	<xsl:otherwise>
	<xsl:call-template name = "piuPadri">
		<xsl:with-param name = "className" select = "name"/>
	</xsl:call-template>
	</xsl:otherwise>
	</xsl:choose>
	
	{
	<xsl:apply-templates select = "properties"/>	
	
	<xsl:call-template name="costruisciAssociazione">   
		<xsl:with-param name = "className" select = "name"/>
    	</xsl:call-template>
	}
	
	<!--<xsl:call-template name="cardinalitaAssociazione">   
		<xsl:with-param name = "className" select = "name"/>
    	</xsl:call-template>		-->
	
	<xsl:call-template name="completeGen"> 
		<xsl:with-param name = "className" select = "name"/>
    	</xsl:call-template>

	<xsl:call-template name = "factPiuPadri">
		<xsl:with-param name = "className" select = "name"/>
	</xsl:call-template>
	
	<xsl:call-template name = "factDisjoint">
		<xsl:with-param name = "className" select = "name"/>
	</xsl:call-template>
	
	assert show<xsl:value-of select = "name"/>{
	no <xsl:value-of select = "name"/>
	} check show<xsl:value-of select = "name"/> for 3
	
</xsl:template>

<!-- associazioni-->
<xsl:template match = "association">
	<xsl:call-template name="costruisciFact">   
			<xsl:with-param name = "assName" select = "@identifierName"/>
    </xsl:call-template>
</xsl:template>
	
<xsl:template name="costruisciAssociazione">
		<xsl:param name="className" />
		<xsl:for-each select = "//association">
			<xsl:if test="contains(substring(@identifierName,string-length(/ccdDiagram/name) + 2), $className)"> 
				<xsl:value-of select = "concat(@identifierName,@id)"/> :
				<xsl:for-each select = "roles/role">
				<xsl:if test = "substring(participantClass,string-length(/ccdDiagram/name) + 2) = $className">
					<xsl:if test = "multiplicity/@minimum &gt; 1 and multiplicity/@maximum &lt; 32767">
						set
					</xsl:if>
					<xsl:if test = "multiplicity/@minimum = 1 and multiplicity/@maximum = 32767">
						some
					</xsl:if>
					<xsl:if test = "multiplicity/@minimum = 0 and multiplicity/@maximum = 1">
						lone
					</xsl:if>
					<xsl:if test = "multiplicity/@minimum = 1 and multiplicity/@maximum = 1">
						one
					</xsl:if>	
				</xsl:if>	
				</xsl:for-each>
				<!-- per ogni ruolo cercare l'altra classe-->
				<xsl:for-each select = "roles/role">
					<xsl:if test = "substring(participantClass,string-length(/ccdDiagram/name) + 2) != $className">
						<xsl:value-of select = "substring(participantClass,string-length(/ccdDiagram/name) + 2)"/>,
					</xsl:if>
				</xsl:for-each>	
			</xsl:if>
		</xsl:for-each>
</xsl:template>
	
<!-- questo metodo va rivisto perche' non parte, e funzionerebbe solo se ogni classe ha al piu' una sola associazione-->	
<xsl:template name="cardinalitaAssociazione">
		<xsl:param name="className" />
		<xsl:for-each select = "//association">
		<xsl:if test = "multiplicity/@minimum &gt; 1 or multiplicity/@maximum &lt; 32767">
			<xsl:for-each select = "roles/role">
				<xsl:if test="substring(participantClass,string-length(/ccdDiagram/name) + 2) =  $className "> 
					<!-- se  il maximum e' diverso da infinito-->	
					{
					<xsl:if test = "multiplicity/@maximum != '32767'">
						#<xsl:value-of select = "//association/@identifierName"/> &lt;= <xsl:value-of select = "multiplicity/@maximum"/>
					</xsl:if>	
					
					<!-- se sia il max che  il min-->	
					<xsl:if test = "multiplicity/@maximum != '32767' and  multiplicity/@minimum != '0' "> &amp;&amp; </xsl:if>
					
					<!-- se  il minimum e' diverso da zero-->	
					<xsl:if test = "multiplicity/@minimum != '0' ">
						#<xsl:value-of select = "//association/@identifierName"/> &gt;= <xsl:value-of select = "multiplicity/@minimum"/> 
					</xsl:if>
					}
				</xsl:if>
		</xsl:for-each>
		</xsl:if>
		</xsl:for-each>
</xsl:template>
	
<!-- fact delle associazioni-->	
<xsl:template name = "costruisciFact">
	<xsl:param name = "assName"/>
		<xsl:for-each select = "//association">
		<xsl:if test = "@identifierName = $assName">
		<xsl:variable name = "numPartecipanti"> <xsl:value-of select = "count(role)" /> </xsl:variable>
			fact <xsl:value-of select = "concat('ass',$assName)"/> { 
			all   			
			<xsl:for-each select = "roles/role">
				<xsl:variable name="listcount">
					<xsl:number count = "roles/role" format="1" /> 
				</xsl:variable>
				<xsl:variable name = "primaClasse">
				<xsl:value-of select = "substring(participantClass,string-length(/ccdDiagram/name) + 2)"/>
				</xsl:variable>
				<xsl:value-of select = "$primaClasse"/>:
				<xsl:value-of select = "$primaClasse"/>
				<xsl:choose>
					<xsl:when test="$listcount = $numPartecipanti + 1"> , </xsl:when>
					<xsl:otherwise> | </xsl:otherwise>
				</xsl:choose>				
			</xsl:for-each>	
			(
			
			<xsl:variable name = "primaClasse">
			<xsl:value-of select = "substring(roles/role/participantClass,string-length(/ccdDiagram/name) + 2)"/>
			</xsl:variable>		

			<xsl:for-each select = "roles/role">	
				<xsl:if test = "substring(participantClass,string-length(/ccdDiagram/name) + 2) != $primaClasse">
				<xsl:value-of select = "$primaClasse"/> in <xsl:value-of select = "substring(participantClass,string-length(/ccdDiagram/name) + 2)"/>.<xsl:value-of select = "concat($assName,../../@id)"/>
				</xsl:if>
			</xsl:for-each>
			
			) &lt;=&gt;
			(
			<xsl:for-each select = "roles/role">	
				<xsl:if test = "substring(participantClass,string-length(/ccdDiagram/name) + 2) != $primaClasse">
				 <xsl:value-of select = "substring(participantClass,string-length(/ccdDiagram/name) + 2)"/> in <xsl:value-of select = "$primaClasse"/>.<xsl:value-of select = "concat($assName,../../@id)"/>
				</xsl:if>
			</xsl:for-each>
			
			
			)
			}
			</xsl:if>
		</xsl:for-each>
		
</xsl:template>


<!--generalizzazioni-->
	
<xsl:template name="costruisciGeneralizzazione">
		<xsl:param name="className"/>
		<xsl:for-each select = "//generalization">
			<xsl:variable name = "padre"> <xsl:value-of select = "baseClass"/> </xsl:variable> <!-- conto quante volte il padre e' un figlio a sua volta-->
			<xsl:variable name = "numPadre"> <xsl:value-of select = "count(//generalization/subClasses[subClass = $padre])"/> </xsl:variable>
				
			<xsl:for-each select = "subClasses/subClass">
				<xsl:if test="substring(.,string-length(/ccdDiagram/name) + 2) = $className"> <!--mettere un call template cn parametri-->
				
					<xsl:choose><!--se il padre e' a sua volta un figlio allora metti in e poi si crea un fact se e' disjoint-->
					<xsl:when test = "$numPadre &lt; 1">
						<xsl:if test = "../../@disjoint = 'true'" >
							extends 
						</xsl:if>
						<xsl:if test = "../../@disjoint = 'false'">
							in
						</xsl:if>
					</xsl:when>
					<xsl:otherwise> in </xsl:otherwise>
					</xsl:choose>				
					<xsl:value-of select = "substring(../../baseClass,string-length(/ccdDiagram/name) + 2)"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>		
</xsl:template>
<!--qui di seguito vengono gestite le gerarchie-->

<xsl:template name = "piuPadri">
	<xsl:param name = "className"/>
	
	sig <xsl:value-of select = "$className"/>
	in
	<xsl:for-each select = "//generalization">
		<xsl:variable name="listcount">
			<xsl:number count = "*[subClasses/subClass = concat(/ccdDiagram/name,'_',$className)]" format="1" /> 
		</xsl:variable>
		
		<xsl:if test = "$listcount = 1">
		<xsl:value-of select = "substring(baseClass,string-length(/ccdDiagram/name) + 2)"/><!---<xsl:value-of select = "$listcount"/>-->
		</xsl:if>

	</xsl:for-each>	
</xsl:template>


<xsl:template name = "factPiuPadri">
<xsl:param name = "className"/>
	<xsl:if test = "count(//generalization/subClasses[subClass = concat(/ccdDiagram/name,'_',$className)]) &gt; 1">
	<xsl:for-each select = "//generalization">
		<xsl:variable name="listcount">
			<xsl:number count = "*[subClasses/subClass = concat(/ccdDiagram/name,'_',$className)]" format="1" /> 
		</xsl:variable>
		
		<xsl:if test = "$listcount &gt; 1">
		fact father<xsl:value-of select = "$className"/>{
		<xsl:value-of select = "$className"/> in 
		<xsl:value-of select = "substring(baseClass,string-length(/ccdDiagram/name) + 2)"/><!---<xsl:value-of select = "$listcount"/>-->
		}
		</xsl:if>

	</xsl:for-each>
	</xsl:if>
</xsl:template>


<xsl:template name = "factDisjoint">
	<xsl:param name = "className"/>
	<xsl:for-each select = "//generalization">
		<xsl:variable name = "padre"> <xsl:value-of select = "baseClass"/> </xsl:variable> <!-- conto quante volte il padre e' un figlio a sua volta-->
		<xsl:variable name = "numPadre"> <xsl:value-of select = "count(//generalization/subClasses[subClass = $padre])"/> </xsl:variable>
		<xsl:if test = "$numPadre &gt; 0 and @disjoint = 'true'">
			<xsl:for-each select = "subClasses/subClass">	
			<xsl:variable name = "sottoClasse"><xsl:value-of select = "substring(.,string-length(/ccdDiagram/name)+2)"/></xsl:variable>
				<xsl:if test = "$sottoClasse = $className">
					<xsl:if test = "position() &lt; count(../subClass)">
					fact disjointfigli{ 
					all <xsl:value-of select = "substring($sottoClasse,0,3)"/>:<xsl:value-of select = "$sottoClasse"/> |
					(<xsl:value-of select = "substring($sottoClasse,0,3)"/> in <xsl:value-of select = "$sottoClasse"/> ) &lt;=&gt;
					(
					<!--<xsl:value-of select = "substring(.,string-length(/ccdDiagram/name)+2)"/>-->
					<xsl:for-each select = "../subClass">
						<xsl:variable name = "listcount">
							<xsl:number count = "subClass" format = "1"/>
						</xsl:variable>
						<xsl:if test = "substring(.,string-length(/ccdDiagram/name)+2) != $className">
							(<xsl:value-of select = "$sottoClasse"/> not in <xsl:value-of select = "substring(.,string-length(/ccdDiagram/name)+2)"/>)
						
							<xsl:choose>
								<xsl:when test = "$listcount &lt; count(../subClass)"> &amp;&amp;</xsl:when>		
								<xsl:otherwise>)</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
					</xsl:for-each>
		
					}
					</xsl:if>
		
				<xsl:if test = "position() = count(../subClass)">
				fact disjointfigli{ 
				all <xsl:value-of select = "substring($sottoClasse,0,3)"/>:<xsl:value-of select = "$sottoClasse"/> |
				(<xsl:value-of select = "substring($sottoClasse,0,3)"/> in <xsl:value-of select = "$sottoClasse"/> ) &lt;=&gt;
				(
				<!--<xsl:value-of select = "substring(.,string-length(/ccdDiagram/name)+2)"/>-->
				<xsl:for-each select = "../subClass">
					<xsl:variable name = "listcount">
						<xsl:number count = "subClass" format = "1"/>
					</xsl:variable>
					<xsl:if test = "substring(.,string-length(/ccdDiagram/name)+2) != $className">
						(<xsl:value-of select = "$sottoClasse"/> not in <xsl:value-of select = "substring(.,string-length(/ccdDiagram/name)+2)"/>)
					
						<xsl:choose>
							<xsl:when test = "$listcount = count(../subClass) - 1">)</xsl:when>		
							<xsl:otherwise>&amp;&amp;</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</xsl:for-each>
				}
				</xsl:if>
				</xsl:if>
		</xsl:for-each>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<!-- qui viene gestita la completezza di una generalizzazione-->

<xsl:template name = "completeGen">
	<xsl:param name="className"/>
		<xsl:for-each select = "//generalization">
			<xsl:if test = "@complete = 'true' and substring(baseClass,string-length(/ccdDiagram/name) + 2) = $className">
				fact partition<xsl:value-of select = "$className"/>{
				<xsl:for-each select = "subClasses">
					<xsl:variable name = "numFigli"> <xsl:value-of select = "count(subClass)" /> </xsl:variable>
					<xsl:for-each select = "subClass">
						<xsl:variable name="listcount">
							<xsl:number count = "subClass" format="1"/> 
						</xsl:variable>
						<xsl:value-of select = "substring(.,string-length(/ccdDiagram/name) + 2)"/>
						<xsl:choose>
							<xsl:when test="$listcount &lt; $numFigli"> + </xsl:when>
							<xsl:otherwise> = </xsl:otherwise>
						</xsl:choose>
					</xsl:for-each>		
				</xsl:for-each>
				<xsl:value-of select = "$className"/>
				}
			</xsl:if>
		</xsl:for-each>	
</xsl:template>


<!-- proprieta' delle classi: attributi e operazioni-->
<xsl:template match = "properties">
	<xsl:apply-templates select = "attribute"/>
	<xsl:apply-templates select = "operation"/>
</xsl:template>		

<xsl:template match = "attribute">
	<xsl:value-of select = "name"/>: 
	
	<xsl:if test = "multiplicity/@minimum = 0 and multiplicity/@maximum = 32767">
		set
	</xsl:if>	
	
	<xsl:if test = "multiplicity/@minimum = 1 and multiplicity/@maximum = 32767">
		some
	</xsl:if>
	
	<xsl:if test = "multiplicity/@minimum = 0 and multiplicity/@maximum = 1">
		lone
	</xsl:if>
	
	<xsl:if test = "multiplicity/@minimum = 1 and multiplicity/@maximum = 1">
		one
	</xsl:if>
	
	<xsl:choose>
	<xsl:when test = "@type = 'Integer &gt; 0'">
		IntegerPos
	</xsl:when>	
	<xsl:when test = "@type = 'Intero &gt;= 0'">
		IntegerNonNeg
	</xsl:when>	
	<xsl:otherwise><xsl:value-of select = "@type"/> </xsl:otherwise>
	</xsl:choose>
	,
</xsl:template>

<xsl:template match = "operation">
	<xsl:value-of select = "name"/>: 
	<!--parametri attenzione alla fine, usare il contatore-->
		<xsl:choose>
		<xsl:when test = "count(inputParameters/parameter) &gt; 0">
		<xsl:for-each select = "inputParameters/parameter">
			<xsl:choose>
				<xsl:when test = "@type = 'Integer &gt; 0'">
					IntegerPos
				</xsl:when>	
				<xsl:when test = "@type = 'Intero &gt;= 0'">
					IntegerNonNeg
				</xsl:when>	
				<xsl:otherwise><xsl:value-of select = "@type"/> </xsl:otherwise>
			</xsl:choose> -> ( 
		</xsl:for-each>
		<xsl:choose>
				<xsl:when test = "returnTypeString = 'Integer &gt; 0'">
					IntegerPos
				</xsl:when>	
				<xsl:when test = "returnTypeString = 'Intero &gt;= 0'">
					IntegerNonNeg
				</xsl:when>	
				<xsl:otherwise><xsl:value-of select = "returnTypeString"/> </xsl:otherwise>
		</xsl:choose>
		
		)
			<xsl:if test = "count(inputParameters/parameter) &gt; 1">) </xsl:if>	
		</xsl:when>
		<xsl:otherwise>
		<xsl:choose>
				<xsl:when test = "returnTypeString = 'Integer &gt; 0'">
					IntegerPos
				</xsl:when>	
				<xsl:when test = "returnTypeString = 'Intero &gt;= 0'">
					IntegerNonNeg
				</xsl:when>	
				<xsl:otherwise><xsl:value-of select = "returnTypeString"/> </xsl:otherwise>
		</xsl:choose>
		</xsl:otherwise>
		</xsl:choose>,
</xsl:template>


<xsl:template name = "tipi">
<!--"String","Integer","Integer > 0","Intero >= 0","Boolean","Float","Character","Byte","Date","DateTime"-->
		<xsl:if test = "count(//attribute[@type = 'String']) &gt;0 or count(//operation[@type = 'String']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'String']) &gt;0 ">
		<xsl:call-template name = "String"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Integer']) &gt;0 or count(//operation[@type = 'Integer']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Integer']) &gt;0 ">
		<xsl:call-template name = "Integer"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Integer &gt; 0']) &gt;0 or count(//operation[@type = 'Integer &gt; 0']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Integer &gt; 0']) &gt;0 ">
		<xsl:call-template name = "IntegerPos"/>
		</xsl:if>
		
		
		<xsl:if test = "count(//attribute[@type = 'Intero &gt;= 0']) &gt;0 or count(//operation[@type = 'Intero &gt;= 0']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Intero &gt;= 0']) &gt;0 ">
		<xsl:call-template name = "IntegerNonNeg"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Float']) &gt;0 or count(//operation[@type = 'Float']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Float']) &gt;0 ">
		<xsl:call-template name = "Float"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Boolean']) &gt;0 or count(//operation[@type = 'Boolean']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Boolean']) &gt;0 ">
		<xsl:call-template name = "Boolean"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Character']) &gt;0 or count(//operation[@type = 'Character']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Character']) &gt;0 ">
		<xsl:call-template name = "Character"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Byte']) &gt;0 or count(//operation[@type = 'Byte']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Byte']) &gt;0 ">
		<xsl:call-template name = "Byte"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'Date']) &gt;0 or count(//operation[@type = 'Date']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'Date']) &gt;0 ">
		<xsl:call-template name = "Date"/>
		</xsl:if>
		
		<xsl:if test = "count(//attribute[@type = 'DateTime']) &gt;0 or count(//operation[@type = 'DateTime']) &gt;0 or count(//operation/inputParameters/parameter[@type = 'DateTime']) &gt;0 ">
		<xsl:call-template name = "DateTime"/>
		</xsl:if>
		
</xsl:template>
<!--"String","Integer","Integer > 0","Intero >= 0","Boolean","Float","Character","Byte","Date","DateTime"-->

<xsl:template name = "String">
sig String{}
</xsl:template>

<xsl:template name = "Integer">
sig Integer{}
</xsl:template>

<xsl:template name = "IntegerPos">
<xsl:if test = "count(//attribute[@type = 'Integer']) = 0 and 
				count(//operation[@type = 'Integer']) = 0 and 
				count(//operation/inputParameters/parameter[@type = 'Integer']) = 0 ">
	<xsl:call-template name = "Integer"/>
</xsl:if>	
sig IntegerPos extends Integer{}
</xsl:template>

<xsl:template name = "IntegerNonNeg">
<xsl:if test = "(count(//attribute[@type = 'Integer']) = 0 and 
				count(//operation[@type = 'Integer']) = 0 and 
				count(//operation/inputParameters/parameter[@type = 'Integer']) = 0)
				and
				(count(//attribute[@type = 'Integer &gt; 0']) = 0 and
				count(//operation[@type = 'Integer &gt; 0']) = 0 and
				count(//operation/inputParameters/parameter[@type = 'Integer &gt; 0']) = 0)">
	<xsl:call-template name = "Integer"/>
</xsl:if>
sig IntegerNonNeg extends Integer{}
</xsl:template>

<xsl:template name = "Float">
sig Float{}
</xsl:template>

<xsl:template name = "Boolean">
sig Boolean{}
</xsl:template>

<xsl:template name = "Character">
sig Character{}
</xsl:template>

<xsl:template name = "Byte">
sig Byte{}
</xsl:template>

<xsl:template name = "Date">
sig Date{}
</xsl:template>

<xsl:template name = "DateTime">
sig DateTime{}
</xsl:template>




</xsl:stylesheet>
