package it.uniroma1.dis.mfis.redia.verifint.control;

import java.io.*;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.ArrayList;
import java.util.Iterator;

import it.uniroma1.dis.mfis.redia.verifint.control._exceptions.ApplicationException;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types.CTDataType;
import it.uniroma1.dis.mfis.redia.verifint.model.Diagram;
import it.uniroma1.dis.mfis.redia.verifint.control.ExecutionProperties;


public class Archive implements Serializable
{
	private static final long serialVersionUID = -4621459021708555168L;
	public static final String FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE =
		"Error in loading file %1s:\n%2s";
	public static final String FILE_SAVING_EXCEPTION_MESSAGE_TEMPLATE =
		"Error in saving file %1s:\n%2s";
	public static final String SAVING_PATH_KEY = "saving_project_path";
	
	protected static Archive instance;
	private Boolean unsaved;
	public static final String DEFAULT_FILE_SYSTEM_STORAGE_PATH = "temp.mfis";

	//private CCDDiagram conceptualClassDiagram;
	private ArrayList<Diagram> diagrams;

	private SortedSet<CTDataType> conceptuaDataTypes;

	protected Archive()
	{
		this.diagrams = new ArrayList<Diagram>();
		//this.conceptualClassDiagram = null;
		this.conceptuaDataTypes = new TreeSet<CTDataType>();
		this.setUnsaved(false);
	}

	public void save(String path) throws ApplicationException
	{
		try
		{
			path = ExecutionProperties.getInstance().getProperty(SAVING_PATH_KEY) + path;
			if(!path.contains(".mfis")) path = path + ".mfis";
			ObjectOutputStream out;
			out = new ObjectOutputStream(new FileOutputStream(path));
			out.writeObject(instance);
			out.close();
			instance.setUnsaved(false);
		}
		catch(IOException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_SAVING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		this.setUnsaved(false);
	}

	public void save() throws ApplicationException {
		this.save(DEFAULT_FILE_SYSTEM_STORAGE_PATH);
	}

	public boolean load(String path) throws ApplicationException
	{
        ObjectInputStream in;
		try
		{
			path = ExecutionProperties.getInstance().getProperty(Archive.SAVING_PATH_KEY) + path;
			in = new ObjectInputStream(new FileInputStream(path));
			instance = (Archive)in.readObject();
			in.close();
			instance.setUnsaved(false);
		}
		catch(FileNotFoundException err)
		{
			return false;
		}
		catch(ClassNotFoundException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		catch(InvalidClassException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		catch(IOException err)
		{
			err.printStackTrace(System.err);
			throw new ApplicationException(
					String.format(
							FILE_LOADING_EXCEPTION_MESSAGE_TEMPLATE, path, err
						)
					);
		}
		return true;
	}

	public boolean load() throws ApplicationException {
		return this.load(DEFAULT_FILE_SYSTEM_STORAGE_PATH);
	}

	public static Archive getInstance()
	{
		if(instance==null) {
			instance = new Archive();
			return instance;
		}
		return instance;
	}

	public void setUnsaved(boolean unsaved) {
		this.unsaved = unsaved;
	}

	public boolean isUnsaved() {
		return this.unsaved;
	}
/*
	CCDDiagram getConceptualClassDiagram() {
		return conceptualClassDiagram;
	}

	protected void setConceptualClassDiagram(
			CCDDiagram conceptualClassDiagram) {
		this.conceptualClassDiagram = conceptualClassDiagram;
	}
*/
	SortedSet<CTDataType> getConceptuaDataTypes() {
		return conceptuaDataTypes;
	}

	protected void setConceptuaDataTypes(SortedSet<CTDataType> conceptDataTypes) {
		this.conceptuaDataTypes = conceptDataTypes;
	}

	//funzioni gianmichele-carlo
	public boolean addDiagram(Diagram d) {
		boolean added = diagrams.add(d);
		if(added) {
			this.setUnsaved(true);
			return true;
		}
		else return false;
	}

	public boolean removeDiagram(String name) {
		Diagram d = findDiagram(name);
		if(d != null) {
			this.setUnsaved(true);
			return diagrams.remove(d);
		}
		else return false;
	}
	
	public Diagram getDiagram(String name) {
		return findDiagram(name);
	}

	public ArrayList<Diagram> getDiagrams() {
		return diagrams;
	}

	public void setDiagrams(ArrayList diags) {
		diagrams = diags;
	}

	private Diagram findDiagram(String name) {
		Diagram ret;
		Iterator it = diagrams.iterator();
		while(it.hasNext()) {
			ret = (Diagram)it.next();
			if(ret.getName().equals(name))
				return ret;
		 }
		return null;
	}
}
