package it.uniroma1.dis.mfis.redia.verifint.control;

import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.SortedSet;
import java.util.TreeSet;
import java.io.*;
import java.util.Scanner;
import java.util.Iterator;

import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._links.CTType_isTypeOf_CTTypeDefined;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams._links.*;
import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.Multiplicity;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types.CTBaseType;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.types._interfaces.CTType;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.*;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control.provers.analysis.classDiagram.CCDAlloy4Prover;

import edu.mit.csail.sdg.alloy4.Err;

public class ClassDiagramController {
	
	private ClassDiagramController(){}

	public static void costruisciClasse(CCDDiagram diagram, String nome, ArrayList attributi, ArrayList operazioni){
			
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();
		CCDSpecialization_Factory specialization_factory = new CCDSpecialization_Factory();
		
		CCDClass classe = class_factory.createInstance(diagram, nome);
		HashMap types = CTBaseType.getBaseTypes();
		Iterator attIt = attributi.iterator();
		while(attIt.hasNext()){//si cercano e creano gli attributi
			String[] row = (String[])attIt.next();
			String tipoStr = row[1];
			CTBaseType type = (CTBaseType)types.get(tipoStr);
			String[] multipRow = row[2].split(",");
			String max = multipRow[1];
			if(max.equals("*")) max = "" + Multiplicity.UNBOUNDED;
			Multiplicity multiplicity = new Multiplicity(Short.parseShort(multipRow[0]),Short.parseShort(max));
			CCDAttribute name_attribute = attribute_factory.createInstance(diagram, classe, type, row[0], multiplicity);
		}
		Iterator opIt = operazioni.iterator();
		while(opIt.hasNext()){//si cercano e creano le operazioni
			String op = (String)opIt.next();
			String params = op.substring(op.indexOf("(")+1,op.indexOf(")"));
			
			String nomeOper = op.substring(0,op.indexOf("("));
			String valRit = op.substring(op.indexOf(")")+ 3);
			CTBaseType retType = (CTBaseType)types.get(valRit);
			CCDOperation oper = op_factory.createInstance(diagram, nomeOper, classe);
			if(params.contains(",")){//operazioni con piu' di un parametro			
				Scanner sc = new Scanner(params);
				sc.useDelimiter(", ");
				while(sc.hasNext()){
					String parametro = sc.next();
					String nomeparam = parametro.substring(0,parametro.indexOf(":"));
					String tipoparam = parametro.substring(parametro.indexOf(":")+2);
					CTBaseType type = (CTBaseType)types.get(tipoparam);
					CCDParameter operparameter = parameter_factory.createInstance(diagram, oper, type, nomeparam);
				}
			}
			else {
				String parametro = params;
				if(params.contains(":")){// operazioni con un parametro
					String nomeparam = parametro.substring(0,parametro.indexOf(":"));
					String tipoparam = parametro.substring(parametro.indexOf(":")+2);
					CTBaseType type = (CTBaseType)types.get(tipoparam);
					CCDParameter operparameter = parameter_factory.createInstance(diagram, oper, type, nomeparam);
				}
			}
			if(retType!= null) CTType_isTypeOf_CTTypeDefined.addLink(retType,oper);	
		}	
	}
	
	public static void costruisciAssociazione(CCDDiagram diagram, String[] classi, String nomeAss, Multiplicity[] card, String[] rules){
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();
		CCDSpecialization_Factory specialization_factory = new CCDSpecialization_Factory();
		
		List componenti = diagram.getComponents();
		Iterator it = componenti.iterator();
		CCDClass classe = null;
		List<CCDAssociation_Factory.Role_TemporaryContainer>  ruoli = new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		while(it.hasNext()){
			CCDComponent comp = (CCDComponent)it.next();
			String nome = comp.getName();
			for(int i = 0; i < classi.length; i++) 
				if(classi[i].equals(nome)) {
					classe = (CCDClass) comp;
					Multiplicity mul = card[i];
					ruoli.add(new CCDAssociation_Factory.Role_TemporaryContainer(rules[i], mul, classe));
				}					
		}
		CCDAssociation_Factory.Association_Roles_Container ruoli_container = association_factory.createInstances(diagram, nomeAss, ruoli);
	}

	public static void costruisciGeneralizzazione(CCDDiagram diagram, String nomePadre, String[] nomiFigli, boolean disjoint, boolean complete){
		CCDClass_Factory class_factory = new CCDClass_Factory();
		CCDOperation_Factory op_factory = new CCDOperation_Factory();
		CCDAttribute_Factory attribute_factory = new CCDAttribute_Factory();
		CCDAssociation_Factory association_factory = new CCDAssociation_Factory();
		CCDParameter_Factory parameter_factory = new CCDParameter_Factory();
		CCDSpecialization_Factory specialization_factory = new CCDSpecialization_Factory();
		
		List componenti = diagram.getComponents();
		Iterator it = componenti.iterator();
		CCDClass padre = null;
		CCDClass figlio = null;
		TreeSet<CCDClass> figli = new TreeSet<CCDClass>();
		List<CCDAssociation_Factory.Role_TemporaryContainer>  ruoli = new ArrayList<CCDAssociation_Factory.Role_TemporaryContainer>(2);
		while(it.hasNext()){
			CCDComponent classe = (CCDComponent)it.next();
			String nome = classe.getName();
			if(nomePadre.equals(nome))
				padre = (CCDClass)classe;
			for(int i = 0; i < nomiFigli.length; i++) 
				if(nomiFigli[i].equals(nome)) {
					figlio = (CCDClass)classe;
					figli.add(figlio);
				}
		}
		CCDGeneralization generalization = new CCDGeneralization_Factory().createInstance(diagram, disjoint, complete, padre, figli);
	
	}

	public static void eliminaClasse(CCDDiagram diagram, String nome){
		List<CCDComponent> elementi = diagram.getComponents();
		CCDClass toDelete = null;
		Iterator itComp = elementi.iterator();
		while(itComp.hasNext()){
			CCDElement el = (CCDElement)itComp.next();
			if(el instanceof CCDClass) {
				CCDClass classe = (CCDClass)el;
				if(classe.getName().equals(nome))
					toDelete = classe;
			}
			if(el instanceof CCDAssociation) {
				CCDAssociation assoc = (CCDAssociation)el;
				List<CCDRole> ruoli = assoc.getRoles();
				String[] partecipanti = new String[ruoli.size()];
				int i = 0;
				Iterator roleIt = ruoli.iterator();
				while(roleIt.hasNext()) {
					CCDRole ruolo = (CCDRole)roleIt.next();
					CCDClass classePartec = ruolo.getParticipantClass();
					partecipanti[i] = classePartec.getName();
					i++;
				}
				boolean trovata = false;
				for(i = 0; i < partecipanti.length; i++) {
					if(nome.contains(partecipanti[i])) {
						trovata = true;
						break;
					}
				}
				if(trovata) 
					eliminaAssociazione(diagram,assoc.getName());
			}
			if(el instanceof CCDGeneralization) {
				CCDGeneralization isa = (CCDGeneralization)el;
				CCDClass father = isa.getBaseClass();
				String nomePadre = father.getName();
				if(nomePadre.equals(nome))
					eliminaGeneralizzazione(diagram,nome,"");
				SortedSet<CCDClass> subclasses = isa.getSubClasses();
				for(CCDClass classeFiglio : subclasses) {
					String nomeFiglio = classeFiglio.getName();		
					if(nomeFiglio.equals(nome))
						eliminaGeneralizzazione(diagram,"",nome);
				}
			}
		}
		if(toDelete != null) CCDDiagram_isComposedBy_CCDElement.removeLink(diagram,toDelete);
	}

	public static void eliminaAssociazione(CCDDiagram diagram, String nome){
		List<CCDComponent> elementi = diagram.getComponents();
		CCDAssociation toDelete = null;
		Iterator itComp = elementi.iterator();
		while(itComp.hasNext()){
			CCDElement el = (CCDElement) itComp.next();
			if(el instanceof CCDAssociation) {			
				CCDAssociation assoc = (CCDAssociation)el;
				if(nome.equals(assoc.getName())) toDelete = assoc;
				else {//prendo i nomi delle classi dell'associazione
					List<CCDRole> ruoli = assoc.getRoles();
					String[] partecipanti = new String[ruoli.size()];
					int i = 0;
					Iterator roleIt = ruoli.iterator();
					while(roleIt.hasNext()) {
						CCDRole ruolo = (CCDRole)roleIt.next();
						CCDClass classe = ruolo.getParticipantClass();
						partecipanti[i] = classe.getName();
						i++;
					}
					boolean trovata = true;
					for(i = 0; i < partecipanti.length; i++) {
						if(!nome.contains(partecipanti[i])) {
							trovata = false;
							break;
						}
					}
					if(trovata) toDelete = assoc;
				}
			}
		}
		if(toDelete != null) CCDDiagram_isComposedBy_CCDElement.removeLink(diagram,toDelete);	
	}

	public static void eliminaGeneralizzazione(CCDDiagram diagram, String padre, String figlio){
		List<CCDComponent> elementi = diagram.getComponents();
		CCDGeneralization toDelete = null;
		Iterator itComp = elementi.iterator();
		while(itComp.hasNext()){
			CCDElement el = (CCDElement) itComp.next();
			if(el instanceof CCDGeneralization) {			
				CCDGeneralization isa = (CCDGeneralization)el;
				CCDClass father = isa.getBaseClass();
				SortedSet<CCDClass> subclasses = isa.getSubClasses();
				String nomePadre = father.getName();
				if(nomePadre.equals(padre)) {
					if(figlio.equals("")) toDelete = isa; //cancello tutte le generalizzazioni di cui  padre
					else { //altrimenti cancella solo quelle di cui il primo figlio e "figlio"
						for(CCDClass classeFiglio : subclasses) {
							String nomeFiglio = classeFiglio.getName();		
							if(nomeFiglio.equals(figlio)) 
								toDelete = isa;
						}
					}
				}
				else {//cancello tutte quelle a cui partecipa come figlio
					for(CCDClass classeFiglio : subclasses) {
						String nomeFiglio = classeFiglio.getName();		
						if(nomeFiglio.equals(figlio)) 
							toDelete = isa;
					}
				}
			}
		}	
		if(toDelete != null) CCDDiagram_isComposedBy_CCDElement.removeLink(diagram,toDelete);
	}

	public static boolean existComponent(CCDDiagram diagram, String name) {
		List componenti = diagram.getComponents();
		Iterator it = componenti.iterator();
		while(it.hasNext()){
			CCDComponent component = (CCDComponent)it.next();
			if(name.equals(component.getName())) return true;
		}
		return false;
	}
	public static boolean existAssociation(CCDDiagram diagram, String nomeAss, String[] classi){
		List componenti = diagram.getComponents();
		Iterator it = componenti.iterator();
		while(it.hasNext()){
			CCDComponent component = (CCDComponent)it.next(); 
			if(component instanceof CCDAssociation){
				CCDAssociation ass = (CCDAssociation) component;
				if(nomeAss.equals(ass.getName()) && !nomeAss.equals("")) 
					return true;
				if(nomeAss.equals(ass.getName()) && nomeAss.equals("")) {
					List <CCDRole> ruoli = ass.getRoles();
					Iterator itrole = ruoli.iterator();
					boolean a = false;
					boolean b = false;
					while(itrole.hasNext()){
						CCDRole ruolo = (CCDRole)itrole.next();
						CCDClass classe = ruolo.getParticipantClass();
						if(classi[0].equals(classe.getName())) 
							a = true;
						if(classi[1].equals(classe.getName())) 
							b = true;	
					}
					if(a && b) return true;
				}
			}
		}
		return false;
	}

	public static ProvingResult verificaFormale(CCDDiagram diagram) throws Err {
		CCDAlloy4Prover prover = new CCDAlloy4Prover(diagram);
		return prover.proveAlloy4(diagram.getName());
	}

	public static String getAlloyInputString(CCDDiagram diagram) throws IOException {
		CCDAlloy4Prover prover = new CCDAlloy4Prover(diagram);
		return prover.getAlloyInput(diagram.getName());
	}
}
		
		
		
	
	

