package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.Multiplicity;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;

import edu.mit.csail.sdg.alloy4.Err;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

public class AssociationListener implements ActionListener {

	private AssociationFrame frame;
	public static final String OK = "OK";
	public static final String ANNULLA = "ANNULLA";
	
	public AssociationListener(AssociationFrame unFrame){
		frame = unFrame;
	}

	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OK) {
			try {
				aggiungiAssociazione();
			}
			catch(UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		}
		if (com == ANNULLA){
			frame.dispose();		
		}
	}
	
	private void aggiungiAssociazione() throws UnspecifiedParameterException, Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String nomeAss = frame.nomeAssTxt.getText();
		String[] classi = new String[2];
		String classe1 = frame.clTxt1.getText();
		String classe2 = frame.clTxt2.getText();
		if(classe1.equals("") || classe2.equals(""))
			throw new UnspecifiedParameterException("Class field empty!");
		if(!ClassDiagramController.existComponent(diagram, classe1) || !ClassDiagramController.existComponent(diagram, classe2))
			throw new UnspecifiedParameterException("Class does not exist!");
		classi[0] = classe1;
		classi[1] = classe2;
		if(ClassDiagramController.existAssociation(diagram,nomeAss,classi))
			throw new UnspecifiedParameterException("Association already exist!\n Change name or abort.");
		boolean simple = frame.simpleBtn.isSelected();
		boolean oriented = frame.orientedBtn.isSelected();
		String moltepMin1 = frame.carMinTxt1.getText();
		String moltepMax1 = frame.carMaxTxt1.getText();
		String moltepMin2 = frame.carMinTxt2.getText();
		String moltepMax2 = frame.carMaxTxt2.getText();
		if(moltepMin1.equals("") || moltepMax1.equals("") || moltepMin2.equals("") || moltepMax2.equals(""))
			throw new UnspecifiedParameterException("Cardinality required!");
		Multiplicity[] multiplicity = new Multiplicity[2];
		try {
			short min1 = Short.parseShort(moltepMin1);
			short min2 = Short.parseShort(moltepMin2);
			short max1, max2;
			if(moltepMax1.equals("*")) max1 = Multiplicity.UNBOUNDED;
			else max1 = Short.parseShort(moltepMax1);
			if(moltepMax2.equals("*")) max2 = Multiplicity.UNBOUNDED;
			else max2 = Short.parseShort(moltepMax2);
			Multiplicity m1 = new Multiplicity(min1,max1);
			Multiplicity m2 = new Multiplicity(min2,max2);
			multiplicity[0] = m1;
			multiplicity[1] = m2;
		}
		catch(NumberFormatException err) {
			JOptionPane.showMessageDialog(frame,"Impossible casting cardinality");
		}
		String[] rules = new String[2];
		rules[0] = frame.ruTxt1.getText();
		rules[1] = frame.ruTxt2.getText();
		creaAssociazione(classi, nomeAss, multiplicity, rules);
		ProvingResult proving = provaFormale();
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}
		disegnaAssociazione(nomeAss, classi, multiplicity, simple, oriented, fileAls, proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");
	}	

	private void creaAssociazione(String[] classi, String nomeAss, Multiplicity[] multiplicity, String[] rules) {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.costruisciAssociazione(diagram, classi, nomeAss, multiplicity, rules);
		archive.setUnsaved(true);
	}

	private void disegnaAssociazione(String nomeAss, String[] classi, Multiplicity[] multiplicity, 
					boolean simple, boolean oriented, String alloyIn, String alloyOut) {
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();

		Frame fr = null;
		Frame primaClasse = null;
		Frame secondaClasse = null;
		for(Drawable draw : drawableObjects) {
			if(draw instanceof Frame) {
				fr = (Frame)draw;
				java.util.List<Figure> figures = fr.getFigure();
				Iterator itList = figures.iterator();
				while(itList.hasNext()) {
					Figure fig = (Figure)itList.next();
					if(fig instanceof Label) {
						Label nomeClasseLbl = (Label)fig;
						String nomeClasse = nomeClasseLbl.getText();
						if(nomeClasse.equals(classi[0]))
							primaClasse = fr;
						if(nomeClasse.equals(classi[1]))
							secondaClasse = fr;
					}
				}
			}
		}
		Edge edge = new Edge(primaClasse,secondaClasse);
		if(simple)
			edge.setStyle(Edge.STYLE_SIMPLEASSOCIATION);
		if(oriented)
			edge.setStyle(Edge.STYLE_HAS);
		String nomeClassi = classi[0] + "-" + classi[1] ; 
		edge.setLabelName(nomeAss);
		edge.setNomeAss(nomeAss);
		edge.setNomeClass(nomeClassi);
		String card1 = multiplicity[0].toString();
		String card2 = multiplicity[1].toString();
		edge.setMultiplicity(0, card1);
		edge.setMultiplicity(1, card2);
		drawableObjects.add(edge);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane);
		tabSelected.repaint();
		String updateNome = nomeAss;
		if(updateNome.equals("")) updateNome = nomeClassi;
		tabSelected.updateComponentsArea(ContentTabPane.ASSOCIATION_TYPE,updateNome);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		frame.dispose();
	}

	private ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}
}
