package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;

import edu.mit.csail.sdg.alloy4.Err;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.io.*;
import javax.swing.JFrame;


public class ClassListener implements ActionListener{

	public static final String OK = "OK";
	public static final String ANNULLA = "ANNULLA";
	

	private ClassFrame frame;

	public ClassListener(ClassFrame unFrame){
		frame = unFrame;
	}

	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OK)
			try {
				aggiungiClasse(); 
			}
			catch (UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		if(com == ANNULLA) 
			frame.dispose();
	}

	private void aggiungiClasse() throws UnspecifiedParameterException, Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String nome = frame.nomeTxt.getText();
		ArrayList attributi = new ArrayList();
		ArrayList operazioni = new ArrayList();
		if(nome == null || nome.equals(""))
			throw new UnspecifiedParameterException("Invalid class name");
		if(ClassDiagramController.existComponent(diagram,nome))
			throw new UnspecifiedParameterException("Class " + nome + " already exists!");
		for(int i = 0; i < frame.attrPnl.table.getRowCount(); i++) {
			String nomeAttributo = (String)frame.attrPnl.table.getValueAt(i,0);
			String tipoAttributo = (String)frame.attrPnl.table.getValueAt(i,1);
			String cardAttributo = (String)frame.attrPnl.table.getValueAt(i,2);
			String visibility = (String)frame.attrPnl.table.getValueAt(i,3);
			if(visibility.equals("public")) visibility = "+";
			else if(visibility.equals("private")) visibility = "-";
			else if(visibility.equals("protected")) visibility = "*";
			else if(visibility.equals("package")) visibility = "#";
			String[] rowAttributo = {nomeAttributo, tipoAttributo, cardAttributo, visibility};
			attributi.add(rowAttributo);
			if(nomeAttributo == null || nomeAttributo.equals(""))
				throw new UnspecifiedParameterException("Invalid attribute name");
		}
		for(int i = 0; i < frame.operPnl.table.getRowCount(); i++) {
				String operazione = (String)frame.operPnl.table.getValueAt(i,0);
				operazioni.add(operazione);
		}
		creaClasse(nome, attributi, operazioni);
		ProvingResult proving = provaFormale();
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}
		disegnaClasse(nome, attributi, operazioni, fileAls, proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");
	}
	
	private void creaClasse(String nome, ArrayList attributi, ArrayList operazioni){
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.costruisciClasse(diagram,nome,attributi,operazioni);
		archive.setUnsaved(true);
	}	

	private void disegnaClasse(String nome, ArrayList attributi, ArrayList operazioni, String alloyIn, String alloyOut)  {
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();
		
		Frame classe = new Frame(new Point(20,20));
		Label nomeClasse = new Label(nome);
		nomeClasse.setFont(RenderingPane.BOLD_FONT);
		nomeClasse.setTextAlignment(Label.ALIGNMENT_CENTER);
		classe.getFigure().add(nomeClasse);
		classe.getFigure().add(new Figure());
		Iterator itAttr = attributi.iterator();
		while(itAttr.hasNext()) {
			String[] rowAttributo = (String[])itAttr.next();
			String attributo = rowAttributo[3] + " " + rowAttributo[0] + ": " + rowAttributo[1] + " (" + rowAttributo[2] + ")";
			classe.getFigure().add(new Label(attributo));
		}
		classe.getFigure().add(new Figure());
		Iterator itOper = operazioni.iterator();
		while(itOper.hasNext()) {
			String operazione = (String)itOper.next();
			if(!operazione.equals("")) classe.getFigure().add(new Label(operazione));
		}
		drawableObjects.add(classe);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane);
		tabSelected.repaint();
		tabSelected.updateComponentsArea(ContentTabPane.CLASS_TYPE,nome);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		frame.dispose();
	}

	private ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}	
}
