package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;

import edu.mit.csail.sdg.alloy4.Err;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

public class DeleteAssListener implements ActionListener {

	private static DeleteAssFrame frame;
	public static final String OK = "OK";
	public static final String ANNULLA = "ANNULLA";
	
	public DeleteAssListener(DeleteAssFrame unFrame){
		frame = unFrame;
	}

	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OK) {
			try {
				eliminaAssociazione();
			}
			catch(UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		}
		if (com == ANNULLA){
			frame.dispose();		
		}
	}
	
	private void eliminaAssociazione() throws UnspecifiedParameterException, Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String nomeAss;
		String assTxt = frame.nomeAssTxt.getText();
		String classTxt1 = frame.clTxt1.getText();
		String classTxt2 = frame.clTxt2.getText();
		if(assTxt.equals("")) {
			if(!ClassDiagramController.existComponent(diagram,classTxt1) || !ClassDiagramController.existComponent(diagram,classTxt2))
				throw new UnspecifiedParameterException("Class does not exists"); 
			String classi = classTxt1.concat(classTxt2);
			nomeAss = classi;
		}
		else {
			nomeAss = assTxt;
			if(!ClassDiagramController.existComponent(diagram,assTxt))
				throw new UnspecifiedParameterException("Association does not exists"); 
		}
		eliminaOggettoAssociazione(nomeAss);
		ProvingResult proving = provaFormale();
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}

		eliminaDisegnoAssociazione(nomeAss,false,fileAls,proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");		
	}
	
	private static void eliminaOggettoAssociazione(String nomeAss) {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.eliminaAssociazione(diagram, nomeAss);
		archive.setUnsaved(true);
	}
			
	private static void eliminaDisegnoAssociazione(String nomeAss, boolean popup, String alloyIn, String alloyOut) {
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();
		String classi = "";
		String label = "";
		Drawable toDelete = null;
		boolean trovataAss = false;
		for(Drawable draw : drawableObjects) {
			if(draw instanceof Edge) {
				Edge ass = (Edge)draw;
				label = ass.getNomeAss();
				classi = ass.getNomeClass();
				String class1 = classi.substring(0,classi.indexOf("-"));
				String class2 = classi.substring(classi.indexOf("-") + 1);
				if(nomeAss.equals(label) || (nomeAss.contains(class1) && nomeAss.contains(class2))) {
					if(ass.getStyle() == Edge.STYLE_SIMPLEASSOCIATION || ass.getStyle() == Edge.STYLE_HAS) {
						toDelete = draw;
						trovataAss = true;
					}
				}
			}
		}
		if(trovataAss) drawableObjects.remove(toDelete);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane);
		tabSelected.repaint();
		Edge edgeToDel = (Edge)toDelete;
		String aux = edgeToDel.getNomeAss();
		if(aux == null) aux = edgeToDel.getNomeClass();
		tabSelected.deleteCompFromArea(ContentTabPane.ASSOCIATION_TYPE,aux);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		if(!popup)frame.dispose();
	}

	private ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}
}
