package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;

import edu.mit.csail.sdg.alloy4.Err;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;


public class DeleteClassListener implements ActionListener{

	public static final String OKDELETE = "OKDELETE";
	public static final String ANNULLA = "ANNULLA";
	
	private static DeleteClassFrame frame;
	
	public DeleteClassListener(DeleteClassFrame unFrame){
		frame = unFrame;
	}

	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OKDELETE) {
			try {
				cancellaClasse();
			}
			catch(UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		}
		if (com == ANNULLA){
			frame.dispose();		
		}
	}

	private void cancellaClasse() throws UnspecifiedParameterException, Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String nomeClasse = frame.nomeTxt.getText();
		boolean popup = false;
		if(!ClassDiagramController.existComponent(diagram,nomeClasse)) throw new UnspecifiedParameterException("Class does not exists");
		cancellaOggettoClasse(nomeClasse);
		ProvingResult proving = provaFormale();
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}
		cancellaDisegnoClasse(nomeClasse,popup,fileAls,proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");
	}

	public static void cancellaOggettoClasse(String nomeClasse) {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.eliminaClasse(diagram, nomeClasse);
		archive.setUnsaved(true);
	}	
	
	public static void cancellaDisegnoClasse(String nomeClasse, boolean popup, String alloyIn, String alloyOut) {
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();

		Drawable classe = null;
		ArrayList<Drawable> associazioni = new ArrayList<Drawable>();
		for(Drawable draw : drawableObjects){
			if(draw instanceof Frame) {
				Frame fr = (Frame)draw;
				java.util.List<Figure> figures = fr.getFigure();
				Figure fig = figures.get(0);
				if(fig instanceof Label) {
					Label nomeClasseLbl = (Label)fig;
					String nomeLabel = nomeClasseLbl.getText();
					if(nomeLabel.equals(nomeClasse))
						classe = draw;
				}
			}
			else if(draw instanceof Edge) {
				Edge ass = (Edge)draw;
				if(ass.getNomeClass().contains(nomeClasse))
					associazioni.add(draw);
			}
		}	
		Iterator it = associazioni.iterator();
		while(it.hasNext()) {
			Drawable d = (Drawable)it.next();
			Edge edge = (Edge)d;
			String edgeName = edge.getNomeAss();
			if(edgeName == null) edgeName = edge.getNomeClass();
			tabSelected.deleteCompFromArea(ContentTabPane.ISA_TYPE,edgeName);
			tabSelected.deleteCompFromArea(ContentTabPane.ASSOCIATION_TYPE,edgeName);
			drawableObjects.remove(edge);
		}
		if(classe != null) drawableObjects.remove(classe);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane);
		tabSelected.repaint();
		tabSelected.deleteCompFromArea(ContentTabPane.CLASS_TYPE,nomeClasse);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		if(!popup) frame.dispose();
	}

	private static ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}
}
