package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;

import edu.mit.csail.sdg.alloy4.Err;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

public class DeleteISAListener implements ActionListener {

	private static DeleteISAFrame frame;
	public static final String OK = "OK";
	public static final String ANNULLA = "ANNULLA";
	
	public DeleteISAListener(DeleteISAFrame unFrame){
		frame = unFrame;
	}

	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OK) {
			try {
				eliminaISA();
			}
			catch(UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		}
		if (com == ANNULLA){
			frame.dispose();		
		}
	}

	private void eliminaISA() throws UnspecifiedParameterException, Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String padre = frame.clTxt1.getText();
		String figlio = frame.clTxt2.getText();
		if(!ClassDiagramController.existComponent(diagram,padre) || !ClassDiagramController.existComponent(diagram,figlio))
			throw new UnspecifiedParameterException("Class does not exists"); 
		eliminaOggettoISA(padre,figlio);
		ProvingResult proving = provaFormale();
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}
		eliminaDisegnoISA(padre,figlio,false,fileAls,proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");
	}

	private static void eliminaOggettoISA(String padre, String figlio){
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.eliminaGeneralizzazione(diagram, padre, figlio);
		archive.setUnsaved(true);
	}	
		
	private static void eliminaDisegnoISA(String padre, String figlio, boolean popup, String alloyIn, String alloyOut) {
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();
		
		Edge isa = null;
		boolean trovatoISA = false;
		Drawable toDelete = null;
		for(Drawable draw : drawableObjects) {
			if(draw instanceof Edge) {
				isa = (Edge)draw;
				if(isa.getNomeClass().contains(padre) && isa.getNomeClass().contains(figlio) ) {
					if(isa.getStyle() == Edge.STYLE_ISA) {
						trovatoISA = true;
						toDelete = draw;
					}
				}
			}
		}
		if(trovatoISA) drawableObjects.remove(toDelete);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane);
		tabSelected.repaint();
		String delArea = padre + "-" + figlio;
		tabSelected.deleteCompFromArea(ContentTabPane.ISA_TYPE,delArea);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		if(!popup)frame.dispose();
	}

	private ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}
}
