package it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui.frames.classDiagram;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import it.uniroma1.dis.mfis.redia.verifint.control.ClassDiagramController;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.classDiagrams.CCDDiagram;

import it.uniroma1.dis.mfis.redia.verifint.view.graphics.gui._guiutils.UnspecifiedParameterException;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Drawable;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Edge;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Figure;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Frame;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.Label;
import it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine.RenderingPane;
import it.uniroma1.dis.mfis.redia.verifint.control.Archive;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;

import edu.mit.csail.sdg.alloy4.Err;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

public class ISAListener implements ActionListener {

	ISAFrame frame;
	public static final String OK = "OK";
	public static final String ANNULLA = "ANNULLA";
	
	public ISAListener(ISAFrame unFrame){
		frame = unFrame;
	}


	public void actionPerformed(ActionEvent e){
		String com = e.getActionCommand();
		if(com == OK) {
			try {
				aggiungiISA();
			}
			catch (UnspecifiedParameterException unspecErr) {
				JOptionPane.showMessageDialog(frame,unspecErr.message);
			}
			catch (Err err) {
				JOptionPane.showMessageDialog(frame,err.toString());
			}
		}
		if (com == ANNULLA){
			frame.dispose();		
		}
	}
	
	private void aggiungiISA() throws UnspecifiedParameterException, Err {	
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		String nomeDiagramma = jtp.getTitleAt(jtp.getSelectedIndex());
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(nomeDiagramma);
		String padre = frame.clTxt1.getText();
		if(padre.equals(""))
			throw new UnspecifiedParameterException("Class field empty!");
		if(!ClassDiagramController.existComponent(diagram, padre))
			throw new UnspecifiedParameterException("Class does not exist!");		
		ArrayList subclasses = new ArrayList();
		boolean fineLista = false;
		boolean almenoUno = false;
		for(int i = 0; i< frame.figliTable.getRowCount() && !fineLista; i++) {
			String figlio = (String)frame.figliTable.getValueAt(i,0);
			if(figlio.equals("")) fineLista = true;
			else {
				almenoUno = true;
				if(!ClassDiagramController.existComponent(diagram, figlio))
					throw new UnspecifiedParameterException("Class does not exist!");
				else subclasses.add(figlio);
			}
		}
		if(!almenoUno) throw new UnspecifiedParameterException("Insert almost one child!");
		boolean disjoint = frame.disjointBox.isSelected();
		boolean complete = frame.completeBox.isSelected();
		String[] figli = new String[subclasses.size()] ;
		subclasses.toArray(figli);
		creaISA(padre,figli,disjoint,complete);
		ProvingResult proving = provaFormale();	
		String fileAls = "";
		try {
			fileAls = ClassDiagramController.getAlloyInputString(diagram);
		}
		catch (IOException e) {
			throw new UnspecifiedParameterException("Cannot read file " + nomeDiagramma + ".als");
		}
		disegnaISA(padre,figli,disjoint,complete,fileAls,proving.proof);
		if(!proving.proven) 
			throw new UnspecifiedParameterException("Unsatisfiable diagram!\nPlease check prover output in the box below");	
	}
	private void creaISA(String padre, String[] figli, boolean disjoint, boolean complete) {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		ClassDiagramController.costruisciGeneralizzazione(diagram,padre,figli,disjoint,complete);
		archive.setUnsaved(true);
	}

	private void disegnaISA(String padre, String[] figli, boolean disjoint, boolean complete, 
				String alloyIn, String alloyOut) throws UnspecifiedParameterException {
		//recupero il tab di cui devo modificare il contenuto
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		ContentTabPane tabSelected = (ContentTabPane)jtp.getSelectedComponent();
		//recupero gli strumenti per modificare gli oggetti disegnati
		HashSet<Drawable> drawableObjects = tabSelected.getDrawableObj();
		RenderingPane renderingPane = tabSelected.getRenderingPane();

		Frame fr = null;
		Frame padreClasse = null;
		ArrayList subclasses = new ArrayList();
		String listafigli = "";
		for(int t = 0; t <figli.length; t++) listafigli = listafigli + figli[t];
		for(Drawable draw : drawableObjects) {
			if(draw instanceof Frame) {
				fr = (Frame)draw;
				java.util.List<Figure> figures = fr.getFigure(); //è specificato il package perchè altrimenti prendeva un altra List
				Iterator itList = figures.iterator();
				
				while(itList.hasNext()) {
					Figure fig = (Figure)itList.next();

					if(fig instanceof Label) {
						Label nomeClasseLbl = (Label)fig;
						String nomeClasse = nomeClasseLbl.getText();
						if(nomeClasse.equals(padre))
							padreClasse = fr;
						for(int i = 0; i< figli.length;i++) {
							if(figli[i].equals(nomeClasse)) 
								subclasses.add(fr);
						}
					}
				}
			}
		}
		Edge edge = new Edge(subclasses,padreClasse);
		String nome = padre + "-" +  listafigli;// per differenziare il padre dai figli
		edge.setNomeClass(nome);
		edge.setStyle(Edge.STYLE_ISA);
		if(disjoint && !complete) edge.setLabelName("{disjoint}");
		if(complete && !disjoint) edge.setLabelName("{complete}");
		if(disjoint && complete) edge.setLabelName("{disjoint,complete}");	
		drawableObjects.add(edge);
		renderingPane.setDrawableObjects(drawableObjects);
		tabSelected.setRenderingPane(renderingPane); 
		tabSelected.repaint();
		tabSelected.updateComponentsArea(ContentTabPane.ISA_TYPE,nome);
		tabSelected.updateAlloyOutArea(alloyOut);
		tabSelected.updateAlloyInArea(alloyIn);
		frame.dispose();
	}

	private ProvingResult provaFormale() throws Err {
		Archive archive = Archive.getInstance();
		JTabbedPane jtp = PrincipaleFrame.getJTP();
		CCDDiagram diagram = (CCDDiagram)archive.getDiagram(jtp.getTitleAt(jtp.getSelectedIndex()));
		return ClassDiagramController.verificaFormale(diagram);
	}
}
